/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.yifu.cloud.plus.v1.yifu.social.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ArrayUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.read.metadata.holder.ReadRowHolder;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yifu.cloud.plus.v1.yifu.archives.entity.TEmployeeInfo;
import com.yifu.cloud.plus.v1.yifu.archives.entity.TEmployeeProject;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.CommonConstants;
import com.yifu.cloud.plus.v1.yifu.common.core.util.*;
import com.yifu.cloud.plus.v1.yifu.common.dapr.util.ArchivesDaprUtil;
import com.yifu.cloud.plus.v1.yifu.common.mybatis.base.BaseEntity;
import com.yifu.cloud.plus.v1.yifu.social.constants.SocialConstants;
import com.yifu.cloud.plus.v1.yifu.social.entity.*;
import com.yifu.cloud.plus.v1.yifu.social.mapper.TSocialFundInfoMapper;
import com.yifu.cloud.plus.v1.yifu.social.service.*;
import com.yifu.cloud.plus.v1.yifu.social.vo.TSocialFundHistoryVo;
import com.yifu.cloud.plus.v1.yifu.social.vo.TSocialFundInfoDetailVo;
import com.yifu.cloud.plus.v1.yifu.social.vo.TSocialFundInfoSearchVo;
import com.yifu.cloud.plus.v1.yifu.social.vo.TSocialFundInfoVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 社保公积金查询表
 *
 * @author fxj
 * @date 2022-07-15 11:38:05
 */
@Log4j2
@RequiredArgsConstructor
@Service
public class TSocialFundInfoServiceImpl extends ServiceImpl<TSocialFundInfoMapper, TSocialFundInfo> implements TSocialFundInfoService {

	private final ArchivesDaprUtil archivesDaprUtil;
	private final TDispatchInfoService tDispatchInfoService;
	private final TSocialLogService tSocialLogService;
	private final TAuditInfoService tAuditInfoService;
	private final TForecastLibraryService tForecastLibraryService;

	/**
	 * 社保公积金查询表简单分页查询
	 *
	 * @param tSocialFundInfo 社保公积金查询表
	 * @return
	 */
	@Override
	public IPage<TSocialFundInfo> getTSocialFundInfoPage(Page<TSocialFundInfo> page, TSocialFundInfoSearchVo tSocialFundInfo) {
		return baseMapper.getTSocialFundInfoPage(page, tSocialFundInfo);
	}

	/**
	 * 社保公积金查询表批量导出
	 *
	 * @return
	 */
	@Override
	public void listExport(HttpServletResponse response, TSocialFundInfoSearchVo searchVo) {
		String fileName = "社保公积金批量导出" + DateUtil.getThisTime() + ".xlsx";
		//获取要导出的列表
		List<TSocialFundInfo> list = new ArrayList<>();
		long count = noPageCountDiy(searchVo);
		ServletOutputStream out = null;
		try {
			out = response.getOutputStream();
			response.setContentType("multipart/form-data");
			response.setCharacterEncoding("utf-8");
			response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
			// 这里 需要指定写用哪个class去写，然后写到第一个sheet，然后文件流会自动关闭
			ExcelWriter excelWriter = EasyExcel.write(out, TSocialFundInfo.class).build();
			int index = 0;
			if (count > CommonConstants.ZERO_INT) {
				for (int i = 0; i <= count; i += CommonConstants.EXCEL_EXPORT_LIMIT) {
					// 获取实际记录
					searchVo.setLimitStart(i);
					searchVo.setLimitEnd(CommonConstants.EXCEL_EXPORT_LIMIT);
					list = noPageDiy(searchVo);
					if (Common.isNotNull(list)) {
						ExcelUtil<TSocialFundInfo> util = new ExcelUtil<>(TSocialFundInfo.class);
						for (TSocialFundInfo vo : list) {
							util.convertEntity(vo, null, null, null);
						}
					}
					if (list != null) {
						WriteSheet writeSheet = EasyExcel.writerSheet("社保公积金查询表" + index).build();
						excelWriter.write(list, writeSheet);
						index++;
						list.clear();
					}
				}
			} else {
				WriteSheet writeSheet = EasyExcel.writerSheet("社保公积金查询表" + index).build();
				excelWriter.write(list, writeSheet);
			}
			out.flush();
			excelWriter.finish();
		} catch (Exception e) {
			log.error("执行异常", e);
		} finally {
			try {
				if (null != out) {
					out.close();
				}
			} catch (IOException e) {
				log.error("执行异常", e);
			}
		}
	}

	@Override
	public List<TSocialFundInfo> noPageDiy(TSocialFundInfoSearchVo searchVo) {
		LambdaQueryWrapper<TSocialFundInfo> wrapper = buildQueryWrapper(searchVo);
		List<String> idList = Common.getList(searchVo.getIds());
		if (Common.isNotNull(idList)) {
			wrapper.in(TSocialFundInfo::getId, idList);
		}
		if (searchVo.getLimitStart() >= 0 && searchVo.getLimitEnd() > 0) {
			wrapper.last(" limit " + searchVo.getLimitStart() + "," + searchVo.getLimitEnd());
		}
		wrapper.orderByDesc(BaseEntity::getCreateTime);
		return baseMapper.selectList(wrapper);
	}

	private Long noPageCountDiy(TSocialFundInfoSearchVo searchVo) {
		LambdaQueryWrapper<TSocialFundInfo> wrapper = buildQueryWrapper(searchVo);
		List<String> idList = Common.getList(searchVo.getIds());
		if (Common.isNotNull(idList)) {
			wrapper.in(TSocialFundInfo::getId, idList);
		}
		return baseMapper.selectCount(wrapper);
	}

	private LambdaQueryWrapper buildQueryWrapper(TSocialFundInfoSearchVo entity) {
		LambdaQueryWrapper<TSocialFundInfo> wrapper = Wrappers.lambdaQuery();
		if (ArrayUtil.isNotEmpty(entity.getCreateTimes())) {
			wrapper.ge(TSocialFundInfo::getCreateTime, entity.getCreateTimes()[0])
					.le(TSocialFundInfo::getCreateTime,
							entity.getCreateTimes()[1]);
		}
		if (Common.isNotNull(entity.getCreateName())) {
			wrapper.eq(TSocialFundInfo::getCreateName, entity.getCreateName());
		}
		return wrapper;
	}

	/**
	 * @param inputStream
	 * @Description: 批量调基
	 * @Author: hgw
	 * @Date: 2022/7/22 11:32
	 * @return: com.yifu.cloud.plus.v1.yifu.common.core.util.R<java.util.List < com.yifu.cloud.plus.v1.yifu.common.core.util.ErrorMessage>>
	 **/
	@Override
	public R<List<ErrorMessage>> importDiy(InputStream inputStream) {
		List<ErrorMessage> errorMessageList = new ArrayList<>();
		ExcelUtil<TSocialFundHistoryVo> util1 = new ExcelUtil<>(TSocialFundHistoryVo.class);
		// 写法2：
		// 匿名内部类 不用额外写一个DemoDataListener
		// 这里 需要指定读用哪个class去读，然后读取第一个sheet 文件流会自动关闭
		try {
			EasyExcel.read(inputStream, TSocialFundInfoVo.class, new ReadListener<TSocialFundHistoryVo>() {
				/**
				 * 单次缓存的数据量
				 */
				public static final int BATCH_COUNT = CommonConstants.BATCH_COUNT;
				/**
				 *临时存储
				 */
				private List<TSocialFundHistoryVo> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

				@Override
				public void invoke(TSocialFundHistoryVo data, AnalysisContext context) {
					ReadRowHolder readRowHolder = context.readRowHolder();
					Integer rowIndex = readRowHolder.getRowIndex();
					data.setRowIndex(rowIndex + 1);
					ErrorMessage errorMessage = util1.checkEntity(data, data.getRowIndex());
					if (Common.isNotNull(errorMessage)) {
						errorMessageList.add(errorMessage);
					} else {
						cachedDataList.add(data);
					}
					if (cachedDataList.size() >= BATCH_COUNT) {
						saveData();
						// 存储完成清理 list
						cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
					}
				}

				@Override
				public void doAfterAllAnalysed(AnalysisContext context) {
					saveData();
				}

				/**
				 * 加上存储数据库
				 */
				private void saveData() {
					log.info("{}条数据，开始存储数据库！", cachedDataList.size());
					importTSocialFundInfo(cachedDataList, errorMessageList);
					log.info("存储数据库成功！");
				}
			}).sheet().doRead();
		} catch (Exception e) {
			log.error(CommonConstants.IMPORT_DATA_ANALYSIS_ERROR, e);
			return R.failed(CommonConstants.IMPORT_DATA_ANALYSIS_ERROR);
		}
		return R.ok(errorMessageList);
	}

	private void importTSocialFundInfo(List<TSocialFundHistoryVo> excelVOList, List<ErrorMessage> errorMessageList) {
		// 个性化校验逻辑
		ErrorMessage errorMsg;
		// 执行数据插入操作 组装
		TSocialFundHistoryVo excel;

		List<String> idCardList = new ArrayList<>();
		for (int i = 0; i < excelVOList.size(); i++) {
			idCardList.add(excelVOList.get(i).getIdCard());
		}
		List<TSocialFundInfo> socialFundInfoList = this.list(Wrappers.<TSocialFundInfo>query().lambda()
				.in(TSocialFundInfo::getEmpIdcard, idCardList));
		Map<String, TSocialFundInfo> socialFundInfoMap = new HashMap<>();
		if (socialFundInfoList != null && !socialFundInfoList.isEmpty()) {
			for (TSocialFundInfo socialFundInfo : socialFundInfoList) {
				socialFundInfoMap.put(socialFundInfo.getEmpIdcard(), socialFundInfo);
			}
			socialFundInfoList.clear();
		}
		TSocialFundInfo oldSocialFundInfo;
		TSocialFundInfo socialFundInfo;
		for (int i = 0; i < excelVOList.size(); i++) {
			excel = excelVOList.get(i);
			socialFundInfo = socialFundInfoMap.get(excel.getIdCard());
			if (socialFundInfo.getEmpName().equals(excel.getName())) {

				// 0：校验数据有效性-TODO

				// 1：更新社保公积金查询-TODO
				//updateSocialFundInfo(socialFundInfo, excel);
				// 2：新增调基记录
				insertExcel(excel);
				// 3：新增变更日志
				oldSocialFundInfo = new TSocialFundInfo();
				BeanUtil.copyProperties(socialFundInfo, oldSocialFundInfo);
				tSocialLogService.saveModificationRecord(CommonConstants.ONE_INT, oldSocialFundInfo.getId(), oldSocialFundInfo, socialFundInfo);
				// 4：同步预估库-TODO
				//tForecastLibraryService.

				errorMessageList.add(new ErrorMessage(excel.getRowIndex(), CommonConstants.SAVE_SUCCESS, CommonConstants.GREEN));
			} else {
				errorMessageList.add(new ErrorMessage(excel.getRowIndex(), SocialConstants.EMP_NAME_ERROR));
			}
		}
	}

	/**
	 * 插入excel bad record
	 */
	private void insertExcel(TSocialFundHistoryVo excel) {
		TSocialFundInfo insert = new TSocialFundInfo();
		BeanUtil.copyProperties(excel, insert);
		this.save(insert);
	}

	@Override
	public TSocialFundInfoDetailVo getDetailById(String id) {
		TSocialFundInfoDetailVo detailVo = new TSocialFundInfoDetailVo();
		TSocialFundInfo socialFundInfo = this.getById(id);
		if (socialFundInfo != null) {
			// 主表信息
			detailVo.setSocialFundInfo(socialFundInfo);
			// 人员信息
			if (Common.isNotNull(socialFundInfo.getEmpId())) {
				R<TEmployeeInfo> employeeInfoR = archivesDaprUtil.getTEmployeeInfoById(socialFundInfo.getEmpId());
				if (employeeInfoR != null && employeeInfoR.getCode() == CommonConstants.SUCCESS) {
					detailVo.setEmployeeInfo(employeeInfoR.getData());
				}
				// 社保所属项目
				if (Common.isNotNull(socialFundInfo.getSettleDomain())) {
					TEmployeeProject employeeProject = new TEmployeeProject();
					employeeProject.setEmpId(socialFundInfo.getEmpId());
					employeeProject.setDeptId(socialFundInfo.getSettleDomain());
					R<TEmployeeProject> employeeProjectR = archivesDaprUtil.getTEmployeeProjectById(employeeProject);
					if (employeeProjectR != null && employeeProjectR.getCode() == CommonConstants.SUCCESS) {
						detailVo.setSocialProject(employeeProjectR.getData());
					}
				}
				// 公积金所属项目
				if (Common.isNotNull(socialFundInfo.getSettleDomainFund())) {
					TEmployeeProject employeeProject = new TEmployeeProject();
					employeeProject.setEmpId(socialFundInfo.getEmpId());
					employeeProject.setDeptId(socialFundInfo.getSettleDomainFund());
					R<TEmployeeProject> employeeProjectR = archivesDaprUtil.getTEmployeeProjectById(employeeProject);
					if (employeeProjectR != null && employeeProjectR.getCode() == CommonConstants.SUCCESS) {
						detailVo.setFundProject(employeeProjectR.getData());
					}
				}
			}
			// 2022-7-21 12:00
			// 倩倩已确认，社保公积金详情里的合同，采用社保/公积金对应的派单的合同信息（去除状态）
			// 另外，基数sheet，采用变更日志，不是调基表。
			// 社保所属合同
			if (Common.isNotNull(socialFundInfo.getSocialId())) {
				TDispatchInfo dispatchInfo = tDispatchInfoService.getOne(Wrappers.<TDispatchInfo>query().lambda()
						.eq(TDispatchInfo::getSocialId, socialFundInfo.getSocialId())
						.eq(TDispatchInfo::getType, CommonConstants.ZERO_STRING)
						.orderByDesc(TDispatchInfo::getCreateTime).last(CommonConstants.LAST_ONE_SQL));
				detailVo.setSocialContract(dispatchInfo);
			}
			// 公积金所属合同
			if (Common.isNotNull(socialFundInfo.getFundId())) {
				TDispatchInfo dispatchInfo = tDispatchInfoService.getOne(Wrappers.<TDispatchInfo>query().lambda()
						.eq(TDispatchInfo::getFundId, socialFundInfo.getFundId())
						.eq(TDispatchInfo::getType, CommonConstants.ZERO_STRING)
						.orderByDesc(TDispatchInfo::getCreateTime).last(CommonConstants.LAST_ONE_SQL));
				detailVo.setFundContract(dispatchInfo);
			}
			// 调基记录
			List<TSocialLog> socialLogList = tSocialLogService.list(Wrappers.<TSocialLog>query().lambda()
					.eq(TSocialLog::getType, CommonConstants.THREE_INT)
					.eq(TSocialLog::getMainId, socialFundInfo.getId())
					.orderByDesc(TSocialLog::getCreateTime));
			detailVo.setLogList(socialLogList);
			// 流程进展明细
			List<TAuditInfo> auditInfoList = tAuditInfoService.list(Wrappers.<TAuditInfo>query().lambda()
					.eq(TAuditInfo::getEmpIdcard, socialFundInfo.getEmpIdcard())
					.orderByDesc(TAuditInfo::getAuditTime));
			detailVo.setAuditList(auditInfoList);
		}
		return detailVo;
	}

}